@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * ClutterContainer: Generic actor container interface.
  * Author: Emmanuele Bassi <ebassi@openedhand.com>
@@ -92,121 +92,92 @@ static void              create_child_meta  (ClutterContainer *container,
                                              ClutterActor     *actor);
 static void              destroy_child_meta (ClutterContainer *container,
                                              ClutterActor     *actor);
+static void              child_notify       (ClutterContainer *container,
+                                             ClutterActor     *child,
+                                             GParamSpec       *pspec);
 
-static void clutter_container_create_child_meta  (ClutterContainer *container,
-                                                  ClutterActor     *actor);
-static void clutter_container_destroy_child_meta (ClutterContainer *container,
-                                                  ClutterActor     *actor);
+typedef ClutterContainerIface   ClutterContainerInterface;
 
+G_DEFINE_INTERFACE (ClutterContainer, clutter_container, G_TYPE_OBJECT);
 
 static void
-clutter_container_base_init (gpointer g_iface)
+clutter_container_default_init (ClutterContainerInterface *iface)
 {
-  static gboolean initialised = FALSE;
+  GType iface_type = G_TYPE_FROM_INTERFACE (iface);
 
-  if (!initialised)
-    {
-      GType iface_type = G_TYPE_FROM_INTERFACE (g_iface);
-      ClutterContainerIface *iface = g_iface;
+  quark_child_meta =
+    g_quark_from_static_string ("clutter-container-child-data");
 
-      initialised = TRUE;
+  /**
+   * ClutterContainer::actor-added:
+   * @container: the actor which received the signal
+   * @actor: the new child that has been added to @container
+   *
+   * The ::actor-added signal is emitted each time an actor
+   * has been added to @container.
+   *
+   * Since: 0.4
+   */
+  container_signals[ACTOR_ADDED] =
+    g_signal_new (I_("actor-added"),
+                  iface_type,
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (ClutterContainerIface, actor_added),
+                  NULL, NULL,
+                  _clutter_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  CLUTTER_TYPE_ACTOR);
+  /**
+   * ClutterContainer::actor-removed:
+   * @container: the actor which received the signal
+   * @actor: the child that has been removed from @container
+   *
+   * The ::actor-removed signal is emitted each time an actor
+   * is removed from @container.
+   *
+   * Since: 0.4
+   */
+  container_signals[ACTOR_REMOVED] =
+    g_signal_new (I_("actor-removed"),
+                  iface_type,
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (ClutterContainerIface, actor_removed),
+                  NULL, NULL,
+                  _clutter_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  CLUTTER_TYPE_ACTOR);
 
-      quark_child_meta =
-        g_quark_from_static_string ("clutter-container-child-data");
+  /**
+   * ClutterContainer::child-notify:
+   * @container: the container which received the signal
+   * @actor: the child that has had a property set
+   * @pspec: the #GParamSpec of the property set
+   *
+   * The ::child-notify signal is emitted each time a property is
+   * being set through the clutter_container_child_set() and
+   * clutter_container_child_set_property() calls.
+   *
+   * Since: 0.8
+   */
+  container_signals[CHILD_NOTIFY] =
+    g_signal_new (I_("child-notify"),
+                  iface_type,
+                  G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
+                  G_STRUCT_OFFSET (ClutterContainerIface, child_notify),
+                  NULL, NULL,
+                  _clutter_marshal_VOID__OBJECT_PARAM,
+                  G_TYPE_NONE, 2,
+                  CLUTTER_TYPE_ACTOR, G_TYPE_PARAM);
 
-      /**
-       * ClutterContainer::actor-added:
-       * @container: the actor which received the signal
-       * @actor: the new child that has been added to @container
-       *
-       * The ::actor-added signal is emitted each time an actor
-       * has been added to @container.
-       *
-       * Since: 0.4
-       */
-      container_signals[ACTOR_ADDED] =
-        g_signal_new (I_("actor-added"),
-                      iface_type,
-                      G_SIGNAL_RUN_FIRST,
-                      G_STRUCT_OFFSET (ClutterContainerIface, actor_added),
-                      NULL, NULL,
-                      clutter_marshal_VOID__OBJECT,
-                      G_TYPE_NONE, 1,
-                      CLUTTER_TYPE_ACTOR);
-      /**
-       * ClutterContainer::actor-removed:
-       * @container: the actor which received the signal
-       * @actor: the child that has been removed from @container
-       *
-       * The ::actor-removed signal is emitted each time an actor
-       * is removed from @container.
-       *
-       * Since: 0.4
-       */
-      container_signals[ACTOR_REMOVED] =
-        g_signal_new (I_("actor-removed"),
-                      iface_type,
-                      G_SIGNAL_RUN_FIRST,
-                      G_STRUCT_OFFSET (ClutterContainerIface, actor_removed),
-                      NULL, NULL,
-                      clutter_marshal_VOID__OBJECT,
-                      G_TYPE_NONE, 1,
-                      CLUTTER_TYPE_ACTOR);
-
-      /**
-       * ClutterContainer::child-notify:
-       * @container: the container which received the signal
-       * @actor: the child that has had a property set. 
-       *
-       * The ::child-notify signal is emitted each time a property is
-       * being set through the clutter_container_child_set() and
-       * clutter_container_child_set_property() calls.
-       *
-       * Since: 0.8
-       */
-      container_signals[CHILD_NOTIFY] =
-        g_signal_new (I_("child-notify"),
-                      iface_type,
-                      G_SIGNAL_RUN_FIRST,
-                      G_STRUCT_OFFSET (ClutterContainerIface, child_notify),
-                      NULL, NULL,
-                      clutter_marshal_VOID__OBJECT_OBJECT_PARAM,
-                      G_TYPE_NONE, 2,
-                      CLUTTER_TYPE_ACTOR, G_TYPE_PARAM);
-
-      iface->child_meta_type    = G_TYPE_INVALID;
-      iface->create_child_meta  = create_child_meta;
-      iface->destroy_child_meta = destroy_child_meta;
-      iface->get_child_meta     = get_child_meta;
-    }
-}
-
-GType
-clutter_container_get_type (void)
-{
-  static GType container_type = 0;
-
-  if (G_UNLIKELY (!container_type))
-    {
-      const GTypeInfo container_info =
-      {
-        sizeof (ClutterContainerIface),
-        clutter_container_base_init,
-        NULL, /* iface_base_finalize */
-      };
-
-      container_type = g_type_register_static (G_TYPE_INTERFACE,
-                                               I_("ClutterContainer"),
-                                               &container_info, 0);
-
-      g_type_interface_add_prerequisite (container_type, G_TYPE_OBJECT);
-    }
-
-  return container_type;
+  iface->child_meta_type    = G_TYPE_INVALID;
+  iface->create_child_meta  = create_child_meta;
+  iface->destroy_child_meta = destroy_child_meta;
+  iface->get_child_meta     = get_child_meta;
+  iface->child_notify       = child_notify;
 }
 
 /**
- * clutter_container_add:
+ * clutter_container_add: (skip)
  * @container: a #ClutterContainer
  * @first_actor: the first #ClutterActor to add
  * @Varargs: %NULL terminated list of actors to add
@@ -243,6 +214,8 @@ clutter_container_add (ClutterContainer *container,
  * @container. You cannot add a #ClutterActor to more than one
  * #ClutterContainer.
  *
+ * Virtual: add
+ *
  * Since: 0.4
  */
 void
@@ -280,7 +253,7 @@ clutter_container_add_actor (ClutterContainer *container,
 }
 
 /**
- * clutter_container_add_valist:
+ * clutter_container_add_valist: (skip)
  * @container: a #ClutterContainer
  * @first_actor: the first #ClutterActor to add
  * @var_args: list of actors to add, followed by %NULL
@@ -308,7 +281,7 @@ clutter_container_add_valist (ClutterContainer *container,
 }
 
 /**
- * clutter_container_remove:
+ * clutter_container_remove: (skip)
  * @container: a #ClutterContainer
  * @first_actor: first #ClutterActor to remove
  * @Varargs: a %NULL-terminated list of actors to remove
@@ -346,6 +319,8 @@ clutter_container_remove (ClutterContainer *container,
  * yourself, using g_object_ref(). When the actor has been removed,
  * the "actor-removed" signal is emitted by @container.
  *
+ * Virtual: remove
+ *
  * Since: 0.4
  */
 void
@@ -382,7 +357,7 @@ clutter_container_remove_actor (ClutterContainer *container,
 }
 
 /**
- * clutter_container_remove_valist:
+ * clutter_container_remove_valist: (skip)
  * @container: a #ClutterContainer
  * @first_actor: the first #ClutterActor to add
  * @var_args: list of actors to remove, followed by %NULL
@@ -424,7 +399,7 @@ get_children_cb (ClutterActor *child,
  *
  * Retrieves all the children of @container.
  *
- * Return value: (element-type Actor) (transfer container): a list
+ * Return value: (element-type Clutter.Actor) (transfer container): a list
  *   of #ClutterActor<!-- -->s. Use g_list_free() on the returned
  *   list when done.
  *
@@ -446,7 +421,7 @@ clutter_container_get_children (ClutterContainer *container)
 /**
  * clutter_container_foreach:
  * @container: a #ClutterContainer
- * @callback: a function to be called for each child
+ * @callback: (scope call): a function to be called for each child
  * @user_data: data to be passed to the function, or %NULL
  *
  * Calls @callback for each child of @container that was added
@@ -479,7 +454,7 @@ clutter_container_foreach (ClutterContainer *container,
 /**
  * clutter_container_foreach_with_internals:
  * @container: a #ClutterContainer
- * @callback: a function to be called for each child
+ * @callback: (scope call): a function to be called for each child
  * @user_data: data to be passed to the function, or %NULL
  *
  * Calls @callback for each child of @container, including "internal"
@@ -519,6 +494,8 @@ clutter_container_foreach_with_internals (ClutterContainer *container,
  *
  * Raises @actor to @sibling level, in the depth ordering.
  *
+ * Virtual: raise
+ *
  * Since: 0.6
  */
 void
@@ -572,6 +549,8 @@ clutter_container_raise_child (ClutterContainer *container,
  *
  * Lowers @actor to @sibling level, in the depth ordering.
  *
+ * Virtual: lower
+ *
  * Since: 0.6
  */
 void
@@ -744,7 +723,6 @@ destroy_child_meta (ClutterContainer *container,
                     ClutterActor     *actor)
 {
   ClutterContainerIface *iface  = CLUTTER_CONTAINER_GET_IFACE (container);
-  GObject               *object = G_OBJECT (container);
 
   if (iface->child_meta_type == G_TYPE_INVALID)
     return;
@@ -781,7 +759,7 @@ clutter_container_get_child_meta (ClutterContainer *container,
   return NULL;
 }
 
-/*
+/**
  * clutter_container_create_child_meta:
  * @container: a #ClutterContainer
  * @actor: a #ClutterActor
@@ -789,12 +767,25 @@ clutter_container_get_child_meta (ClutterContainer *container,
  * Creates the #ClutterChildMeta wrapping @actor inside the
  * @container, if the #ClutterContainerIface::child_meta_type
  * class member is not set to %G_TYPE_INVALID.
+ *
+ * This function is only useful when adding a #ClutterActor to
+ * a #ClutterContainer implementation outside of the
+ * #ClutterContainer::add() virtual function implementation.
+ *
+ * Applications should not call this function.
+ *
+ * Since: 1.2
  */
-static void
+void
 clutter_container_create_child_meta (ClutterContainer *container,
                                      ClutterActor     *actor)
 {
-  ClutterContainerIface *iface = CLUTTER_CONTAINER_GET_IFACE (container);
+  ClutterContainerIface *iface;
+
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  iface = CLUTTER_CONTAINER_GET_IFACE (container);
 
   if (iface->child_meta_type == G_TYPE_INVALID)
     return;
@@ -805,19 +796,32 @@ clutter_container_create_child_meta (ClutterContainer *container,
     iface->create_child_meta (container, actor);
 }
 
-/*
+/**
  * clutter_container_destroy_child_meta:
  * @container: a #ClutterContainer
  * @actor: a #ClutterActor
  *
  * Destroys the #ClutterChildMeta wrapping @actor inside the
  * @container, if any.
+ *
+ * This function is only useful when removing a #ClutterActor to
+ * a #ClutterContainer implementation outside of the
+ * #ClutterContainer::add() virtual function implementation.
+ *
+ * Applications should not call this function.
+ *
+ * Since: 1.2
  */
-static void
+void
 clutter_container_destroy_child_meta (ClutterContainer *container,
                                       ClutterActor     *actor)
 {
-  ClutterContainerIface *iface = CLUTTER_CONTAINER_GET_IFACE (container);
+  ClutterContainerIface *iface;
+
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  iface = CLUTTER_CONTAINER_GET_IFACE (container);
 
   if (iface->child_meta_type == G_TYPE_INVALID)
     return;
@@ -872,7 +876,7 @@ clutter_container_class_find_child_property (GObjectClass *klass,
  *
  * Returns an array of #GParamSpec for all child properties.
  *
- * Return value: (array length=n_properties) (transfer container): an array
+ * Return value: (array length=n_properties) (transfer full): an array
  *   of #GParamSpec<!-- -->s which should be freed after use.
  *
  * Since: 0.8
@@ -903,6 +907,13 @@ clutter_container_class_list_child_properties (GObjectClass *klass,
   return retval;
 }
 
+static void
+child_notify (ClutterContainer *container,
+              ClutterActor     *actor,
+              GParamSpec       *pspec)
+{
+}
+
 static inline void
 container_set_child_property (ClutterContainer *container,
                               ClutterActor     *actor,
@@ -910,14 +921,15 @@ container_set_child_property (ClutterContainer *container,
                               GParamSpec       *pspec)
 {
   ClutterChildMeta *data;
-  ClutterContainerIface *iface;
 
   data = clutter_container_get_child_meta (container, actor);
   g_object_set_property (G_OBJECT (data), pspec->name, value);
 
-  iface = CLUTTER_CONTAINER_GET_IFACE (container);
-  if (G_LIKELY (iface->child_notify))
-    iface->child_notify (container, actor, pspec);
+  g_signal_emit (container, container_signals[CHILD_NOTIFY],
+                 (pspec->flags & G_PARAM_STATIC_NAME)
+                   ? g_quark_from_static_string (pspec->name)
+                   : g_quark_from_string (pspec->name),
+                 actor, pspec);
 }
 
 /**
@@ -1020,8 +1032,10 @@ clutter_container_child_set (ClutterContainer *container,
           break;
         }
 
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, var_args, 0, &error);
+      G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
+                            var_args, 0,
+                            &error);
+
       if (error)
         {
           /* we intentionally leak the GValue because it might
@@ -1184,3 +1198,31 @@ clutter_container_child_get (ClutterContainer *container,
 
   va_end (var_args);
 }
+
+/**
+ * clutter_container_child_notify:
+ * @container: a #ClutterContainer
+ * @child: a #ClutterActor
+ * @pspec: a #GParamSpec
+ *
+ * Calls the <function>child_notify()</function> virtual function of
+ * #ClutterContainer. The default implementation will emit the
+ * #ClutterContainer::child-notify signal.
+ *
+ * Since: 1.6
+ */
+void
+clutter_container_child_notify (ClutterContainer *container,
+                                ClutterActor     *child,
+                                GParamSpec       *pspec)
+{
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (child));
+  g_return_if_fail (pspec != NULL);
+
+  g_return_if_fail (clutter_actor_get_parent (child) == CLUTTER_ACTOR (container));
+
+  CLUTTER_CONTAINER_GET_IFACE (container)->child_notify (container,
+                                                         child,
+                                                         pspec);
+}
