@@ -98,18 +98,19 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
+#include "clutter-actor.h"
+#include "clutter-backend-private.h"
+#include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-event.h"
-#include "clutter-backend.h"
+#include "clutter-feature.h"
+#include "clutter-frame-source.h"
 #include "clutter-main.h"
 #include "clutter-master-clock.h"
-#include "clutter-feature.h"
-#include "clutter-actor.h"
-#include "clutter-stage.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
-#include "clutter-version.h" 	/* For flavour define */
-#include "clutter-frame-source.h"
 #include "clutter-profile.h"
+#include "clutter-stage-private.h"
+#include "clutter-version.h" 	/* For flavour define */
 
 #include "cogl/cogl.h"
 #include "pango/cogl-pango.h"
@@ -134,7 +135,7 @@ static gboolean clutter_enable_accessibility = TRUE;
 
 static guint clutter_default_fps             = 60;
 
-static PangoDirection clutter_text_direction = CLUTTER_TEXT_DIRECTION_LTR;
+static ClutterTextDirection clutter_text_direction = CLUTTER_TEXT_DIRECTION_LTR;
 
 static guint clutter_main_loop_level         = 0;
 static GSList *main_loops                    = NULL;
@@ -181,7 +182,7 @@ static const GDebugKey clutter_paint_debug_keys[] = {
   { "disable-clipped-redraws", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS },
   { "redraws", CLUTTER_DEBUG_REDRAWS },
   { "paint-volumes", CLUTTER_DEBUG_PAINT_VOLUMES },
-  { "disable-culling", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS }
+  { "disable-culling", CLUTTER_DEBUG_DISABLE_CULLING }
 };
 
 #ifdef CLUTTER_ENABLE_PROFILE
@@ -235,7 +236,7 @@ _clutter_do_redraw (ClutterStage *stage)
 
   ctx = _clutter_context_get_default ();
 
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE);
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
   _clutter_stage_reset_picks_per_frame_counter (stage);
 
   _clutter_backend_ensure_context (ctx->backend, stage);
@@ -343,6 +344,26 @@ _clutter_id_to_color (guint id, ClutterColor *col)
 
   ctx = _clutter_context_get_default ();
 
+  if (ctx->fb_g_mask == 0)
+    {
+      /* Figure out framebuffer masks used for pick */
+      cogl_get_bitmasks (&ctx->fb_r_mask,
+			 &ctx->fb_g_mask,
+			 &ctx->fb_b_mask, NULL);
+
+      ctx->fb_r_mask_used = ctx->fb_r_mask;
+      ctx->fb_g_mask_used = ctx->fb_g_mask;
+      ctx->fb_b_mask_used = ctx->fb_b_mask;
+
+      /* XXX - describe what "fuzzy picking" is */
+      if (clutter_use_fuzzy_picking)
+	{
+	  ctx->fb_r_mask_used--;
+	  ctx->fb_g_mask_used--;
+	  ctx->fb_b_mask_used--;
+	}
+    }
+
   /* compute the numbers we'll store in the components */
   red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
         & (0xff >> (8-ctx->fb_r_mask_used));
@@ -555,7 +576,7 @@ _clutter_do_pick (ClutterStage   *stage,
   /* It's possible that we currently have a static scene and have renderered a
    * full, unclipped pick buffer. If so we can simply continue to read from
    * this cached buffer until the scene next changes. */
-  if (_clutter_stage_get_pick_buffer_valid (stage))
+  if (_clutter_stage_get_pick_buffer_valid (stage, mode))
     {
       CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
       cogl_read_pixels (x, y, 1, 1,
@@ -632,12 +653,11 @@ _clutter_do_pick (ClutterStage   *stage,
       if (G_LIKELY (!(clutter_pick_debug_flags &
                       CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
         cogl_clip_pop ();
+
+      _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
     }
   else
-    _clutter_stage_set_pick_buffer_valid (stage, TRUE);
-
-  /* Make sure Cogl flushes any batched geometry to the GPU driver */
-  cogl_flush ();
+    _clutter_stage_set_pick_buffer_valid (stage, TRUE, mode);
 
   /* Read the color of the screen co-ords pixel. RGBA_8888_PRE is used
      even though we don't care about the alpha component because under
@@ -1025,15 +1045,8 @@ clutter_threads_set_lock_functions (GCallback enter_fn,
   clutter_threads_unlock = leave_fn;
 }
 
-typedef struct
-{
-  GSourceFunc func;
-  gpointer data;
-  GDestroyNotify notify;
-} ClutterThreadsDispatch;
-
-static gboolean
-clutter_threads_dispatch (gpointer data)
+gboolean
+_clutter_threads_dispatch (gpointer data)
 {
   ClutterThreadsDispatch *dispatch = data;
   gboolean ret = FALSE;
@@ -1048,8 +1061,8 @@ clutter_threads_dispatch (gpointer data)
   return ret;
 }
 
-static void
-clutter_threads_dispatch_free (gpointer data)
+void
+_clutter_threads_dispatch_free (gpointer data)
 {
   ClutterThreadsDispatch *dispatch = data;
 
@@ -1176,8 +1189,8 @@ clutter_threads_add_idle_full (gint           priority,
   dispatch->notify = notify;
 
   return g_idle_add_full (priority,
-                          clutter_threads_dispatch, dispatch,
-                          clutter_threads_dispatch_free);
+                          _clutter_threads_dispatch, dispatch,
+                          _clutter_threads_dispatch_free);
 }
 
 /**
@@ -1252,8 +1265,8 @@ clutter_threads_add_timeout_full (gint           priority,
 
   return g_timeout_add_full (priority,
                              interval,
-                             clutter_threads_dispatch, dispatch,
-                             clutter_threads_dispatch_free);
+                             _clutter_threads_dispatch, dispatch,
+                             _clutter_threads_dispatch_free);
 }
 
 /**
@@ -1281,86 +1294,6 @@ clutter_threads_add_timeout (guint       interval,
                                            NULL);
 }
 
-/**
- * clutter_threads_add_frame_source_full:
- * @priority: the priority of the frame source. Typically this will be in the
- *            range between #G_PRIORITY_DEFAULT and #G_PRIORITY_HIGH.
- * @fps: the number of times per second to call the function
- * @func: function to call
- * @data: data to pass to the function
- * @notify: function to call when the timeout source is removed
- *
- * Sets a function to be called at regular intervals holding the Clutter
- * threads lock, with the given priority. The function is called repeatedly
- * until it returns %FALSE, at which point the timeout is automatically
- * removed and the function will not be called again. The @notify function
- * is called when the timeout is removed.
- *
- * This function is similar to clutter_threads_add_timeout_full()
- * except that it will try to compensate for delays. For example, if
- * @func takes half the interval time to execute then the function
- * will be called again half the interval time after it finished. In
- * contrast clutter_threads_add_timeout_full() would not fire until a
- * full interval after the function completes so the delay between
- * calls would be @interval * 1.5. This function does not however try
- * to invoke the function multiple times to catch up missing frames if
- * @func takes more than @interval ms to execute.
- *
- * See also clutter_threads_add_idle_full().
- *
- * Rename to: clutter_threads_add_frame_source
- *
- * Return value: the ID (greater than 0) of the event source.
- *
- * Since: 0.8
- */
-guint
-clutter_threads_add_frame_source_full (gint           priority,
-				       guint          fps,
-				       GSourceFunc    func,
-				       gpointer       data,
-				       GDestroyNotify notify)
-{
-  ClutterThreadsDispatch *dispatch;
-
-  g_return_val_if_fail (func != NULL, 0);
-
-  dispatch = g_slice_new (ClutterThreadsDispatch);
-  dispatch->func = func;
-  dispatch->data = data;
-  dispatch->notify = notify;
-
-  return clutter_frame_source_add_full (priority,
-					fps,
-					clutter_threads_dispatch, dispatch,
-					clutter_threads_dispatch_free);
-}
-
-/**
- * clutter_threads_add_frame_source: (skip)
- * @fps: the number of times per second to call the function
- * @func: function to call
- * @data: data to pass to the function
- *
- * Simple wrapper around clutter_threads_add_frame_source_full().
- *
- * Return value: the ID (greater than 0) of the event source.
- *
- * Since: 0.8
- */
-guint
-clutter_threads_add_frame_source (guint       fps,
-				  GSourceFunc func,
-				  gpointer    data)
-{
-  g_return_val_if_fail (func != NULL, 0);
-
-  return clutter_threads_add_frame_source_full (G_PRIORITY_DEFAULT,
-						fps,
-						func, data,
-						NULL);
-}
-
 /**
  * clutter_threads_enter:
  *
@@ -1606,21 +1539,6 @@ clutter_init_real (GError **error)
 
   clutter_text_direction = clutter_get_text_direction ();
 
-  /* Figure out framebuffer masks used for pick */
-  cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
-
-  ctx->fb_r_mask_used = ctx->fb_r_mask;
-  ctx->fb_g_mask_used = ctx->fb_g_mask;
-  ctx->fb_b_mask_used = ctx->fb_b_mask;
-
-  /* XXX - describe what "fuzzy picking" is */
-  if (clutter_use_fuzzy_picking)
-    {
-      ctx->fb_r_mask_used--;
-      ctx->fb_g_mask_used--;
-      ctx->fb_b_mask_used--;
-    }
-
   /* Initiate event collection */
   _clutter_backend_init_events (ctx->backend);
 
@@ -1875,7 +1793,7 @@ clutter_get_option_group (void)
  * calling g_option_context_parse() with the #GOptionGroup returned by this
  * function requires a subsequent explicit call to clutter_init(); use this
  * function when needing to set foreign display connection with
- * clutter_x11_set_display(), or with gtk_clutter_init().
+ * clutter_x11_set_display(), or with <function>gtk_clutter_init()</function>.
  *
  * Return value: (transfer full): a #GOptionGroup for the commandline arguments
  *   recognized by Clutter
