@@ -18,11 +18,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
 #ifndef __CLUTTER_SCRIPT_H__
 #define __CLUTTER_SCRIPT_H__
 
@@ -83,9 +85,24 @@ typedef enum {
   CLUTTER_SCRIPT_ERROR_INVALID_VALUE
 } ClutterScriptError;
 
+/**
+ * CLUTTER_SCRIPT_ERROR:
+ *
+ * Error domain for the #ClutterScript errors
+ *
+ * Since: 0.6
+ */
 #define CLUTTER_SCRIPT_ERROR    (clutter_script_error_quark ())
 GQuark clutter_script_error_quark (void);
 
+/**
+ * ClutterScript:
+ *
+ * The #ClutterScript structure contains only private data
+ * and should be accessed using the provided API
+ *
+ * Since: 0.6
+ */
 struct _ClutterScript
 {
   /*< private >*/
@@ -94,6 +111,17 @@ struct _ClutterScript
   ClutterScriptPrivate *priv;
 };
 
+/**
+ * ClutterScriptClass:
+ * @get_type_from_name: virtual function used to map a type name
+ *   to a #GType. This function should only be overridden by
+ *   language bindings in order to map native types to #GType.
+ *   The default implementation is equivalent to g_type_from_name()
+ *
+ * The #ClutterScriptClass structure contains only private data
+ *
+ * Since: 0.6
+ */
 struct _ClutterScriptClass
 {
   /*< private >*/
@@ -125,11 +153,14 @@ guint          clutter_script_load_from_data       (ClutterScript  *script,
                                                     const gchar    *data,
                                                     gssize          length,
                                                     GError        **error);
+
 GObject *      clutter_script_get_object           (ClutterScript  *script,
                                                     const gchar    *name);
 gint           clutter_script_get_objects          (ClutterScript  *script,
                                                     const gchar    *first_name,
                                                     ...) G_GNUC_NULL_TERMINATED;
+GList *        clutter_script_list_objects         (ClutterScript  *script);
+
 void           clutter_script_unmerge_objects      (ClutterScript  *script,
                                                     guint           merge_id);
 void           clutter_script_ensure_objects       (ClutterScript  *script);
