@@ -35,16 +35,12 @@
 #include "cogl-texture-private.h"
 #include "cogl-texture-2d-private.h"
 #include "cogl-texture-driver.h"
-#include "cogl-material-opengl-private.h"
+#include "cogl-pipeline-opengl-private.h"
 #include "cogl-debug.h"
 
 #include <stdlib.h>
 
-#ifdef HAVE_COGL_GLES2
-
-#include "../gles/cogl-gles2-wrapper.h"
-
-#else /* HAVE_COGL_GLES2 */
+#ifndef HAVE_COGL_GLES2
 
 #define glGenFramebuffers                 ctx->drv.pf_glGenFramebuffers
 #define glBindFramebuffer                 ctx->drv.pf_glBindFramebuffer
@@ -67,6 +63,10 @@
 #define GL_FRAMEBUFFER_COMPLETE 0x8CD5
 #endif
 
+static void _cogl_atlas_free (CoglAtlas *atlas);
+
+COGL_OBJECT_INTERNAL_DEFINE (Atlas, atlas);
+
 /* If we want to do mulitple blits from a texture (such as when
    reorganizing the atlas) then it's quicker to download all of the
    data once and upload multiple times from that. This struct is used
@@ -213,12 +213,14 @@ _cogl_atlas_new (CoglPixelFormat texture_format,
   atlas->texture_format = texture_format;
   _cogl_callback_list_init (&atlas->reorganize_callbacks);
 
-  return atlas;
+  return _cogl_atlas_object_new (atlas);
 }
 
-void
+static void
 _cogl_atlas_free (CoglAtlas *atlas)
 {
+  COGL_NOTE (ATLAS, "%p: Atlas destroyed", atlas);
+
   if (atlas->texture)
     cogl_handle_unref (atlas->texture);
   if (atlas->map)
@@ -487,7 +489,8 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
         _cogl_rectangle_map_get_remaining_space (atlas->map) *
         100 / (_cogl_rectangle_map_get_width (atlas->map) *
                _cogl_rectangle_map_get_height (atlas->map));
-      COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+      COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+                 atlas,
                  _cogl_rectangle_map_get_width (atlas->map),
                  _cogl_rectangle_map_get_height (atlas->map),
                  _cogl_rectangle_map_get_n_rectangles (atlas->map),
@@ -557,7 +560,7 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
   /* If we can't create a map with the texture then give up */
   if (new_map == NULL)
     {
-      COGL_NOTE (ATLAS, "Could not fit texture in the atlas");
+      COGL_NOTE (ATLAS, "%p: Could not fit texture in the atlas", atlas);
       ret = FALSE;
     }
   /* We need to migrate the existing textures into a new texture */
@@ -566,7 +569,7 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
              _cogl_rectangle_map_get_width (new_map),
              _cogl_rectangle_map_get_height (new_map))) == COGL_INVALID_HANDLE)
     {
-      COGL_NOTE (ATLAS, "Could not create a CoglTexture2D");
+      COGL_NOTE (ATLAS, "%p: Could not create a CoglTexture2D", atlas);
       _cogl_rectangle_map_free (new_map);
       ret = FALSE;
     }
@@ -577,7 +580,8 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
       _cogl_atlas_notify_reorganize (atlas);
 
       COGL_NOTE (ATLAS,
-                 "Atlas %s with size %ix%i",
+                 "%p: Atlas %s with size %ix%i",
+                 atlas,
                  atlas->map == NULL ||
                  _cogl_rectangle_map_get_width (atlas->map) !=
                  _cogl_rectangle_map_get_width (new_map) ||
@@ -614,7 +618,8 @@ _cogl_atlas_reserve_space (CoglAtlas             *atlas,
                100 / (_cogl_rectangle_map_get_width (atlas->map) *
                       _cogl_rectangle_map_get_height (atlas->map)));
 
-      COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+      COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+                 atlas,
                  _cogl_rectangle_map_get_width (atlas->map),
                  _cogl_rectangle_map_get_height (atlas->map),
                  _cogl_rectangle_map_get_n_rectangles (atlas->map),
@@ -634,10 +639,12 @@ _cogl_atlas_remove (CoglAtlas *atlas,
 {
   _cogl_rectangle_map_remove (atlas->map, rectangle);
 
-  COGL_NOTE (ATLAS, "Removed rectangle sized %ix%i",
+  COGL_NOTE (ATLAS, "%p: Removed rectangle sized %ix%i",
+             atlas,
              rectangle->width,
              rectangle->height);
-  COGL_NOTE (ATLAS, "Atlas is %ix%i, has %i textures and is %i%% waste",
+  COGL_NOTE (ATLAS, "%p: Atlas is %ix%i, has %i textures and is %i%% waste",
+             atlas,
              _cogl_rectangle_map_get_width (atlas->map),
              _cogl_rectangle_map_get_height (atlas->map),
              _cogl_rectangle_map_get_n_rectangles (atlas->map),
