@@ -32,8 +32,9 @@
 
 #ifdef COGL_ENABLE_DEBUG
 
-/* XXX: If you add a debug option, please also scroll down to
- * cogl_arg_debug_cb() and add a "help" description of the option too.
+/* XXX: If you add a debug option, please also add an option
+ * definition to cogl-debug-options.h. This will enable us - for
+ * example - to emit a "help" description for the option.
  */
 
 /* NB: Only these options get enabled if COGL_DEBUG=all is
@@ -68,8 +69,14 @@ static const GDebugKey cogl_behavioural_debug_keys[] = {
   { "disable-atlas", COGL_DEBUG_DISABLE_ATLAS },
   { "disable-texturing", COGL_DEBUG_DISABLE_TEXTURING},
   { "disable-arbfp", COGL_DEBUG_DISABLE_ARBFP},
+  { "disable-fixed", COGL_DEBUG_DISABLE_FIXED},
   { "disable-glsl", COGL_DEBUG_DISABLE_GLSL},
-  { "disable-blending", COGL_DEBUG_DISABLE_BLENDING}
+  { "disable-blending", COGL_DEBUG_DISABLE_BLENDING},
+  { "disable-npot-textures", COGL_DEBUG_DISABLE_NPOT_TEXTURES},
+  { "wireframe", COGL_DEBUG_WIREFRAME},
+  { "disable-software-clip", COGL_DEBUG_DISABLE_SOFTWARE_CLIP},
+  { "disable-program-caches", COGL_DEBUG_DISABLE_PROGRAM_CACHES},
+  { "disable-fast-read-pixel", COGL_DEBUG_DISABLE_FAST_READ_PIXEL}
 };
 static const int n_cogl_behavioural_debug_keys =
   G_N_ELEMENTS (cogl_behavioural_debug_keys);
@@ -151,9 +158,9 @@ cogl_arg_no_debug_cb (const char *key,
 static GOptionEntry cogl_args[] = {
 #ifdef COGL_ENABLE_DEBUG
   { "cogl-debug", 0, 0, G_OPTION_ARG_CALLBACK, cogl_arg_debug_cb,
-    N_("COGL debugging flags to set"), "FLAGS" },
+    N_("Cogl debugging flags to set"), "FLAGS" },
   { "cogl-no-debug", 0, 0, G_OPTION_ARG_CALLBACK, cogl_arg_no_debug_cb,
-    N_("COGL debugging flags to unset"), "FLAGS" },
+    N_("Cogl debugging flags to unset"), "FLAGS" },
 #endif /* COGL_ENABLE_DEBUG */
   { NULL, },
 };
@@ -184,8 +191,8 @@ cogl_get_option_group (void)
   GOptionGroup *group;
 
   group = g_option_group_new ("cogl",
-                              _("COGL Options"),
-                              _("Show COGL options"),
+                              _("Cogl Options"),
+                              _("Show Cogl options"),
                               NULL, NULL);
 
   g_option_group_set_parse_hooks (group, pre_parse_hook, NULL);
