@@ -252,9 +252,16 @@ _cogl_pipeline_texture_storage_change_notify (CoglHandle texture)
        * we continue to check the rest */
     }
 }
+
 static void
 set_glsl_program (GLuint gl_program)
 {
+#ifdef HAVE_COGL_GLES
+
+  g_return_if_reached ();
+
+#else /* HAVE_COGL_GLES */
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (ctx->current_gl_program != gl_program)
@@ -272,6 +279,8 @@ set_glsl_program (GLuint gl_program)
           ctx->current_gl_program = 0;
         }
     }
+
+#endif /* HAVE_COGL_GLES */
 }
 
 void
@@ -674,11 +683,11 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
       return FALSE;
     }
 
-  if (layers_difference & COGL_PIPELINE_LAYER_STATE_TEXTURE)
+  if (layers_difference & COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA)
     {
+      unsigned long state = COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA;
       CoglPipelineLayer *authority =
-        _cogl_pipeline_layer_get_authority (layer,
-                                            COGL_PIPELINE_LAYER_STATE_TEXTURE);
+        _cogl_pipeline_layer_get_authority (layer, state);
       CoglHandle texture;
       GLuint     gl_texture;
       GLenum     gl_target;
@@ -909,7 +918,8 @@ compare_layer_differences_cb (CoglPipelineLayer *layer, void *user_data)
    * then we force an update of the texture state...
    */
   if (unit->texture_storage_changed)
-    state->layer_differences[state->i] |= COGL_PIPELINE_LAYER_STATE_TEXTURE;
+    state->layer_differences[state->i] |=
+      COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA;
 
   state->i++;
 
@@ -1063,7 +1073,8 @@ _cogl_pipeline_flush_gl_state (CoglPipeline *pipeline,
     {
       /* Bail out asap if we've been asked to re-flush the already current
        * pipeline and we can see the pipeline hasn't changed */
-      if (ctx->current_pipeline_age == pipeline->age)
+      if (ctx->current_pipeline_age == pipeline->age &&
+          ctx->current_pipeline_skip_gl_color == skip_gl_color)
         goto done;
 
       pipelines_difference = ctx->current_pipeline_changes_since_flush;
