@@ -42,7 +42,7 @@
 #include "cogl-texture-private.h"
 #include "cogl-primitives-private.h"
 #include "cogl-private.h"
-#include "cogl-vertex-attribute-private.h"
+#include "cogl-attribute-private.h"
 #include "tesselator/tesselator.h"
 
 #include <string.h>
@@ -233,7 +233,7 @@ _cogl_path_stroke_nodes (CoglPath *path)
     {
       node = &g_array_index (data->path_nodes, CoglPathNode, path_start);
 
-      cogl_draw_vertex_attributes (COGL_VERTICES_MODE_LINE_STRIP,
+      cogl_draw_attributes (COGL_VERTICES_MODE_LINE_STRIP,
                                    0, node->path_size,
                                    data->stroke_vbo_attributes[path_num],
                                    NULL);
@@ -338,7 +338,7 @@ _cogl_path_fill_nodes (CoglPath *path)
 
   _cogl_path_build_fill_vbo (path);
 
-  _cogl_draw_indexed_vertex_attributes_array
+  _cogl_draw_indexed_attributes_array
                                  (COGL_VERTICES_MODE_TRIANGLES,
                                   0, /* first_vertex */
                                   path->data->fill_vbo_n_indices,
@@ -396,9 +396,9 @@ _cogl_add_path_to_stencil_buffer (CoglPath *path,
            will have set up a scissor for the minimum bounding box of
            all of the clips. That box will likely mean that this
            _cogl_clear won't need to clear the entire
-           buffer. _cogl_clear is used instead of cogl_clear because
+           buffer. _cogl_clear4f is used instead of cogl_clear because
            it won't try to flush the journal */
-        _cogl_clear (NULL, COGL_BUFFER_BIT_STENCIL);
+        _cogl_clear4f (COGL_BUFFER_BIT_STENCIL, 0, 0, 0, 0);
       else
         {
           /* Just clear the bounding box */
@@ -459,17 +459,21 @@ _cogl_add_path_to_stencil_buffer (CoglPath *path,
 void
 cogl2_path_fill (CoglPath *path)
 {
+  CoglFramebuffer *framebuffer;
+
   g_return_if_fail (cogl_is_path (path));
 
   if (path->data->path_nodes->len == 0)
     return;
 
-  _cogl_journal_flush ();
+  framebuffer = _cogl_get_framebuffer ();
+
+  _cogl_framebuffer_flush_journal (framebuffer);
 
   /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
    * as the pipeline state) when flushing the clip stack, so should
    * always be done first when preparing to draw. */
-  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
+  _cogl_framebuffer_flush_state (framebuffer, 0);
 
   _cogl_path_fill_nodes (path);
 }
@@ -1443,19 +1447,19 @@ _cogl_path_build_fill_vbo (CoglPath *path)
   g_array_free (tess.vertices, TRUE);
 
   data->fill_vbo_attributes[0] =
-    cogl_vertex_attribute_new (data->fill_vbo,
+    cogl_attribute_new (data->fill_vbo,
                                "cogl_position_in",
                                sizeof (CoglPathTesselatorVertex),
                                G_STRUCT_OFFSET (CoglPathTesselatorVertex, x),
                                2, /* n_components */
-                               COGL_VERTEX_ATTRIBUTE_TYPE_FLOAT);
+                               COGL_ATTRIBUTE_TYPE_FLOAT);
   data->fill_vbo_attributes[1] =
-    cogl_vertex_attribute_new (data->fill_vbo,
+    cogl_attribute_new (data->fill_vbo,
                                "cogl_tex_coord0_in",
                                sizeof (CoglPathTesselatorVertex),
                                G_STRUCT_OFFSET (CoglPathTesselatorVertex, s),
                                2, /* n_components */
-                               COGL_VERTEX_ATTRIBUTE_TYPE_FLOAT);
+                               COGL_ATTRIBUTE_TYPE_FLOAT);
   /* NULL terminator */
   data->fill_vbo_attributes[2] = NULL;
 
@@ -1484,9 +1488,8 @@ _cogl_path_build_stroke_vbo (CoglPath *path)
                                             sizeof (floatVec2),
                                             NULL);
 
-  vbo_p = cogl_buffer_map (COGL_BUFFER (data->stroke_vbo),
-                           COGL_BUFFER_ACCESS_WRITE,
-                           COGL_BUFFER_MAP_HINT_DISCARD);
+  vbo_p =
+    _cogl_buffer_map_for_fill_or_fallback (COGL_BUFFER (data->stroke_vbo));
 
   /* Copy the vertices in and count the number of sub paths. Each sub
      path will form a separate attribute so we can paint the disjoint
@@ -1506,9 +1509,9 @@ _cogl_path_build_stroke_vbo (CoglPath *path)
       n_attributes++;
     }
 
-  cogl_buffer_unmap (COGL_BUFFER (data->stroke_vbo));
+  _cogl_buffer_unmap_for_fill_or_fallback (COGL_BUFFER (data->stroke_vbo));
 
-  data->stroke_vbo_attributes = g_new (CoglVertexAttribute *, n_attributes);
+  data->stroke_vbo_attributes = g_new (CoglAttribute *, n_attributes);
 
   /* Now we can loop the sub paths again to create the attributes */
   for (i = 0, path_start = 0;
@@ -1518,12 +1521,12 @@ _cogl_path_build_stroke_vbo (CoglPath *path)
       node = &g_array_index (data->path_nodes, CoglPathNode, path_start);
 
       data->stroke_vbo_attributes[i] =
-        cogl_vertex_attribute_new (data->stroke_vbo,
-                                   "cogl_position_in",
-                                   sizeof (floatVec2),
-                                   path_start * sizeof (floatVec2),
-                                   2, /* n_components */
-                                   COGL_VERTEX_ATTRIBUTE_TYPE_FLOAT);
+        cogl_attribute_new (data->stroke_vbo,
+                            "cogl_position_in",
+                            sizeof (floatVec2),
+                            path_start * sizeof (floatVec2),
+                            2, /* n_components */
+                            COGL_ATTRIBUTE_TYPE_FLOAT);
     }
 
   data->stroke_vbo_n_attributes = n_attributes;
